# /*
#  * Copyright 2024 KylinSoft Co., Ltd.
#  *
#  * This program is free software: you can redistribute it and/or modify it under
#  * the terms of the GNU General Public License as published by the Free Software
#  * Foundation, either version 3 of the License, or (at your option) any later
#  * version.
#  *
#  * This program is distributed in the hope that it will be useful, but WITHOUT
#  * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
#  * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
#  * details.
#  *
#  * You should have received a copy of the GNU General Public License along with
#  * this program. If not, see <https://www.gnu.org/licenses/>.
#  */

from concurrent import futures
import grpc
import document_pb2_grpc
import document_pb2
from async_cache import AsyncCache
import logging
import json
import os
import multiprocessing
from logging.handlers import TimedRotatingFileHandler
from datetime import datetime, timedelta

logger = logging.getLogger('parse_service_logger')
logger.setLevel(logging.INFO)
formatter = logging.Formatter("%(asctime)s - %(levelname)s - %(message)s")

console_handler = logging.StreamHandler()
console_handler.setLevel(logging.INFO)
console_handler.setFormatter(formatter)
logger.addHandler(console_handler)

logger.info("Logging is set up. Starting the service...")

class DocumentService(document_pb2_grpc.DocumentServiceServicer):
    def __init__(self, max_workers=4):
        self.max_workers = max_workers
        self.async_cache = AsyncCache()
        
    @staticmethod
    def parse_and_chunk_file(file_path, chunker_type, chunk_size,callbake_data):
        from parser_factory import ParserFactory
        from chunker_factory import ChunkerFactory
        """文件解析和分块逻辑，供进程池执行。"""
        try:
            parser = ParserFactory.create_parser(os.path.splitext(file_path)[1][1:])
            parsed_content = parser.parser(file_path)
            chunker = ChunkerFactory.create_chunker(chunker_type)
            chunks = chunker.segment_tokenize(parsed_content, chunk_size)
            return (file_path, chunks,callbake_data)
        except Exception as e:
            logger.error(f"Error parsing or chunking {file_path}: {str(e)}")
            return (file_path, [],callbake_data)

    def SyncChunkDocument(self, request, context):
        """同步文档分块请求，使用进程池并行处理多个文件。"""
        logger.info(f"Received sync request for files: {request.file_paths}")
        tasks = [(file_path, request.chunker_types[i], request.chunk_sizes[i], None) for i, file_path in enumerate(request.file_paths)]
        results = self.run_in_process_pool(self.parse_and_chunk_file, tasks, self.max_workers)
        response = document_pb2.ChunkedResponse(
            response_data=[]
        )
        for file_path, chunk, _ in results:
            response.response_data.append(document_pb2.ResponseData(chunks = chunk, file_path = file_path))
        logger.debug(f"Returning sync response: {response}")
        logger.info(f"chunk_over: {request.file_paths}")
        return response

    def AsyncChunkDocument(self, request, context):
        """异步文档分块请求，使用进程池并行处理多个文件。"""
        logger.info(f"Received async request for files: {request.file_paths}")
        logger.info(f"Request file paths: {request.file_paths}")
        logger.info(f"Request chunker types: {list(request.chunker_types)}")
        logger.info(f"Request chunk sizes: {list(request.chunk_sizes)}")
        logger.info(f"Request callback data: {request.callback_data}")
        tasks = [
            (file_path, request.chunker_types[i], request.chunk_sizes[i],request.callback_data[0])
            for i, file_path in enumerate(request.file_paths)
        ]
        logger.info(f"Prepared tasks: {tasks}")

        def background_process(tasks, context):
            with multiprocessing.Pool(processes=self.max_workers) as pool:
                async_results = [pool.apply_async(self.parse_and_chunk_file, task) for task in tasks]
                all_chunks = []
                for async_result in async_results:
                    try:
                        result = async_result.get(timeout=60)
                        (file_path, chunks, callback_data) = result
                        all_chunks.extend(chunks)
                    except Exception as e:
                        logger.error(f"Error processing task: {e}")
                        all_chunks.append(f"Error: {str(e)}")
                
                response = document_pb2.ChunkedResponse(
                    file_paths=request.file_paths,
                    chunks=[str(chunk) for chunk in all_chunks],
                    callback_data=str(request.callback_data[0])
                )
                return response

        def run_background_process():
            response = background_process(tasks, context)
            context.set_code(grpc.StatusCode.OK)
            context.set_details(json.dumps({"status": "completed"}))
            return response
        response_future = self.async_cache.cache_result(run_background_process, key=request.file_paths)
        return response_future.result()

    def run_in_process_pool(self, target, tasks, max_workers):
        with futures.ThreadPoolExecutor(max_workers=max_workers) as executor:
            return list(executor.map(lambda args: target(*args), tasks))

def serve():
    uid = os.getuid()
    home_directory = os.path.expanduser("~")
    print(f"Home directory: {home_directory}")
    server = grpc.server(futures.ThreadPoolExecutor(max_workers=10),
                options=[('grpc.max_send_message_length', 1024 * 1024 * 1024),
                        ('grpc.max_receive_message_length', 1024 * 1024 * 1024)])
    document_pb2_grpc.add_DocumentServiceServicer_to_server(DocumentService(), server)
    unix_socket_path_sync = '/tmp/document_service_sync_' + str(uid) +'.sock'
    unix_socket_path_async = '/tmp/document_service_async_' + str(uid) +'.sock'
    try:
        os.unlink(unix_socket_path_sync)
    except OSError:
        if os.path.exists(unix_socket_path_sync):
            raise
    
    try:
        os.unlink(unix_socket_path_async)
    except OSError:
        if os.path.exists(unix_socket_path_async):
            raise
    server.add_insecure_port(f'unix://{unix_socket_path_sync}')
    server.add_insecure_port(f'unix://{unix_socket_path_async}')
    server.start()
    logger.info("Server started and listening on Unix domain sockets.")
    server.wait_for_termination()

if __name__ == '__main__':
    serve()



# /*
#  * Copyright 2024 KylinSoft Co., Ltd.
#  *
#  * This program is free software: you can redistribute it and/or modify it under
#  * the terms of the GNU General Public License as published by the Free Software
#  * Foundation, either version 3 of the License, or (at your option) any later
#  * version.
#  *
#  * This program is distributed in the hope that it will be useful, but WITHOUT
#  * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
#  * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
#  * details.
#  *
#  * You should have received a copy of the GNU General Public License along with
#  * this program. If not, see <https://www.gnu.org/licenses/>.
#  */

import json
from .base_parser import BaseParser

class JSONParser(BaseParser):

    def parser(self, file_path) -> str:
        with open(file_path, 'r', encoding='utf-8') as file:
            data = json.load(file)
            return data


if __name__ == '__main__':
    image_parser = JSONParser()
    text = image_parser.parser('/home/kylin/wangyan/test-search/用户手册.json')
    print(text)



# /*
#  * Copyright 2024 KylinSoft Co., Ltd.
#  *
#  * This program is free software: you can redistribute it and/or modify it under
#  * the terms of the GNU General Public License as published by the Free Software
#  * Foundation, either version 3 of the License, or (at your option) any later
#  * version.
#  *
#  * This program is distributed in the hope that it will be useful, but WITHOUT
#  * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
#  * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
#  * details.
#  *
#  * You should have received a copy of the GNU General Public License along with
#  * this program. If not, see <https://www.gnu.org/licenses/>.
#  */

from base_chunker import BaseChunker
from textchunker import TextChunker

class ChunkerFactory:
    def __init__(self) -> None:
        pass

    @staticmethod
    def create_chunker(format_type: str) -> BaseChunker:
        return TextChunker()



# /*
#  * Copyright 2024 KylinSoft Co., Ltd.
#  *
#  * This program is free software: you can redistribute it and/or modify it under
#  * the terms of the GNU General Public License as published by the Free Software
#  * Foundation, either version 3 of the License, or (at your option) any later
#  * version.
#  *
#  * This program is distributed in the hope that it will be useful, but WITHOUT
#  * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
#  * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
#  * details.
#  *
#  * You should have received a copy of the GNU General Public License along with
#  * this program. If not, see <https://www.gnu.org/licenses/>.
#  */

from PIL import Image
import pytesseract
from .base_parser import BaseParser

"""
图片解析器,暂时不用
"""
class ImageParser(BaseParser):

    def parser(self, file_path) -> str:
        image = Image.open(file_path)
        text = pytesseract.image_to_string(image, lang='chi_sim')
        return text


if __name__ == '__main__':
    image_parser = ImageParser()
    text = image_parser.parser('/home/kylin/图片/2024-09-10_13-59-37.bmp')
    print(text)



# /*
#  * Copyright 2024 KylinSoft Co., Ltd.
#  *
#  * This program is free software: you can redistribute it and/or modify it under
#  * the terms of the GNU General Public License as published by the Free Software
#  * Foundation, either version 3 of the License, or (at your option) any later
#  * version.
#  *
#  * This program is distributed in the hope that it will be useful, but WITHOUT
#  * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
#  * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
#  * details.
#  *
#  * You should have received a copy of the GNU General Public License along with
#  * this program. If not, see <https://www.gnu.org/licenses/>.
#  */

from docx import Document
import time
from .base_parser import BaseParser

"""
docx模块
"""
class DOCXParser(BaseParser):

    def parser_docx(self, file_path) -> str:
        docx = Document(file_path)
        content_list = []
        null_text = str(time.time())
        # 解析普通段落
        for paragraph in docx.paragraphs:
            content_list.append(paragraph.text)

        # 遍历文档中的所有表格，并提取表格中的文本
        for table in docx.tables:
            for row in table.rows:
                row_temp = []
                for cell in row.cells:
                    if cell.text != null_text:
                        row_temp.append(cell.text)
                        cell.text = null_text
                joined_row = ','.join(row_temp)
                content_list.append(joined_row)
        result = "".join(content_list)
        return [result]

    def parser(self, file_path) -> str:
        """
        解析docx文件
        """
        return self.parser_docx(file_path)


if __name__ == '__main__':
    DocxParser = DOCXParser()
    paras = DocxParser.parser('/home/wangyan/操作系统大会演示素材v0.2/AIPC出货规模分析预测.docx')
    print(paras)



# /*
#  * Copyright 2024 KylinSoft Co., Ltd.
#  *
#  * This program is free software: you can redistribute it and/or modify it under
#  * the terms of the GNU General Public License as published by the Free Software
#  * Foundation, either version 3 of the License, or (at your option) any later
#  * version.
#  *
#  * This program is distributed in the hope that it will be useful, but WITHOUT
#  * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
#  * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
#  * details.
#  *
#  * You should have received a copy of the GNU General Public License along with
#  * this program. If not, see <https://www.gnu.org/licenses/>.
#  */

import sys
sys.path.append('/usr/lib/kylin-ai-parser-extention/')

from .base_parser import BaseParser
import importlib.util

class OfficeParser(BaseParser):
    def __init__(self) -> None:
        self.module_available = self._module_exists('libparser_extension')

        if self.module_available:
            import libparser_extension
            self.object = libparser_extension.ParserExtension()
        else:
            print("Warning: The required module 'libparser_extension' is not found. Parsing functionality will be limited.")

    def _module_exists(self, module_name: str) -> bool:
        """
        检查指定的模块是否存在。
        """
        spec = importlib.util.find_spec(module_name)
        return spec is not None

    def parser(self, file_path) -> list:
        """
        解析 .doc, .wps, .ppt 文件
        """
        if not self.module_available:
            print("Error: Cannot parse the file because 'libparser_extension' is not available.")
            return []  # 返回空列表作为默认值

        try:
            result = self.object.parserOffice(file_path)
            content = result.split('\r')
            texts = " ".join(content)
            return texts
        except Exception as e:
            print(f"Error during parsing: {e}")
            raise


if __name__ == "__main__":
    parser = OfficeParser()
    result = parser.parser('/home/wangyan/操作系统大会演示素材v0.2/111.wps')
    print(result)


# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: document.proto
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\x0e\x64ocument.proto\x12\x08\x64ocument\"e\n\x0c\x43hunkRequest\x12\x12\n\nfile_paths\x18\x01 \x03(\t\x12\x15\n\rchunker_types\x18\x02 \x03(\t\x12\x13\n\x0b\x63hunk_sizes\x18\x03 \x03(\x05\x12\x15\n\rcallback_data\x18\x04 \x03(\t\"1\n\x0cResponseData\x12\x11\n\tfile_path\x18\x01 \x01(\t\x12\x0e\n\x06\x63hunks\x18\x02 \x03(\t\"@\n\x0f\x43hunkedResponse\x12-\n\rresponse_data\x18\x01 \x03(\x0b\x32\x16.document.ResponseData2\xee\x01\n\x0f\x44ocumentService\x12\x46\n\x11SyncChunkDocument\x12\x16.document.ChunkRequest\x1a\x19.document.ChunkedResponse\x12G\n\x12\x41syncChunkDocument\x12\x16.document.ChunkRequest\x1a\x19.document.ChunkedResponse\x12J\n\x13StreamChunkDocument\x12\x16.document.ChunkRequest\x1a\x19.document.ChunkedResponse0\x01\x62\x06proto3')



_CHUNKREQUEST = DESCRIPTOR.message_types_by_name['ChunkRequest']
_RESPONSEDATA = DESCRIPTOR.message_types_by_name['ResponseData']
_CHUNKEDRESPONSE = DESCRIPTOR.message_types_by_name['ChunkedResponse']
ChunkRequest = _reflection.GeneratedProtocolMessageType('ChunkRequest', (_message.Message,), {
  'DESCRIPTOR' : _CHUNKREQUEST,
  '__module__' : 'document_pb2'
  # @@protoc_insertion_point(class_scope:document.ChunkRequest)
  })
_sym_db.RegisterMessage(ChunkRequest)

ResponseData = _reflection.GeneratedProtocolMessageType('ResponseData', (_message.Message,), {
  'DESCRIPTOR' : _RESPONSEDATA,
  '__module__' : 'document_pb2'
  # @@protoc_insertion_point(class_scope:document.ResponseData)
  })
_sym_db.RegisterMessage(ResponseData)

ChunkedResponse = _reflection.GeneratedProtocolMessageType('ChunkedResponse', (_message.Message,), {
  'DESCRIPTOR' : _CHUNKEDRESPONSE,
  '__module__' : 'document_pb2'
  # @@protoc_insertion_point(class_scope:document.ChunkedResponse)
  })
_sym_db.RegisterMessage(ChunkedResponse)

_DOCUMENTSERVICE = DESCRIPTOR.services_by_name['DocumentService']
if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  _CHUNKREQUEST._serialized_start=28
  _CHUNKREQUEST._serialized_end=129
  _RESPONSEDATA._serialized_start=131
  _RESPONSEDATA._serialized_end=180
  _CHUNKEDRESPONSE._serialized_start=182
  _CHUNKEDRESPONSE._serialized_end=246
  _DOCUMENTSERVICE._serialized_start=249
  _DOCUMENTSERVICE._serialized_end=487
# @@protoc_insertion_point(module_scope)



# /*
#  * Copyright 2024 KylinSoft Co., Ltd.
#  *
#  * This program is free software: you can redistribute it and/or modify it under
#  * the terms of the GNU General Public License as published by the Free Software
#  * Foundation, either version 3 of the License, or (at your option) any later
#  * version.
#  *
#  * This program is distributed in the hope that it will be useful, but WITHOUT
#  * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
#  * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
#  * details.
#  *
#  * You should have received a copy of the GNU General Public License along with
#  * this program. If not, see <https://www.gnu.org/licenses/>.
#  */

import subprocess
import os
from .base_parser import BaseParser
from .pptx_parser import PPTXParser

"""
ppt文件解析器,由于包的依赖问题，暂时不用
"""
class PPTParser(BaseParser):

    def convert_ppt_to_pptx(self, file_path):
        pptx_file_path = file_path.replace(".ppt", ".pptx")

        subprocess.run(["unoconv", "-f", "pptx", "--output", pptx_file_path, file_path], check=True)
        print(f"文件已成功从 {file_path} 转换为 {pptx_file_path}")

        return pptx_file_path

    def parser(self, file_path) -> str:
        pptx_file_path = self.convert_ppt_to_pptx(file_path)
        pptx_parser = PPTXParser()
        pptx_content = pptx_parser.parser(pptx_file_path)

        os.remove(pptx_file_path)
        return pptx_content


if __name__ == "__main__":
    ppt = PPTParser()
    text = ppt.parser("/home/kylin/wangyan/可信设计方法.ppt")



# /*
#  * Copyright 2024 KylinSoft Co., Ltd.
#  *
#  * This program is free software: you can redistribute it and/or modify it under
#  * the terms of the GNU General Public License as published by the Free Software
#  * Foundation, either version 3 of the License, or (at your option) any later
#  * version.
#  *
#  * This program is distributed in the hope that it will be useful, but WITHOUT
#  * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
#  * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
#  * details.
#  *
#  * You should have received a copy of the GNU General Public License along with
#  * this program. If not, see <https://www.gnu.org/licenses/>.
#  */

from bs4 import BeautifulSoup, Comment
from .base_parser import BaseParser

class XMLParser(BaseParser):

    def parser(self, file_path) -> list:
        # 读取文件内容
        with open(file_path, 'r', encoding='utf-8') as file:
            content = file.read()

        soup = BeautifulSoup(content, 'lxml-xml')  # 使用 XML 解析器
        texts = []
        for element in soup.find_all(text=True):
            # 处理普通文本节点
            stripped = element.strip()
            if stripped:
                texts.append(stripped)
        
        # 合并并清理文本
        clean_text = ' '.join(texts)
        clean_text = ''.join(clean_text.split())  # 去除多余空格
        return [clean_text]

if __name__ == '__main__':
    file_path = '/home/kylin/wangyan/test-search/touchgesture.xml'
    html_parser = XMLParser()
    html_content = html_parser.parser(file_path)
    # print(html_parser.parser_tag(file_path, 'keys'))
    print(html_content)


# /*
#  * Copyright 2024 KylinSoft Co., Ltd.
#  *
#  * This program is free software: you can redistribute it and/or modify it under
#  * the terms of the GNU General Public License as published by the Free Software
#  * Foundation, either version 3 of the License, or (at your option) any later
#  * version.
#  *
#  * This program is distributed in the hope that it will be useful, but WITHOUT
#  * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
#  * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
#  * details.
#  *
#  * You should have received a copy of the GNU General Public License along with
#  * this program. If not, see <https://www.gnu.org/licenses/>.
#  */

from pptx import Presentation
from .base_parser import BaseParser

class PPTXParser(BaseParser):

    def parser(self, file_path) -> dict:
        return self.parser_pptx(file_path)

    def parser_pptx(self, file_path) -> str:
        """
        解析PPTX文件，返回文本内容。
        :param file_path: PPTX文件路径
        :return: 文本内容
        """
        prs = Presentation(file_path)
        content = []

        for slide in prs.slides:
            slide_content = self._parse_slide(slide)
            if slide_content == ['']:
                continue
            else:
                content.extend(slide_content)
        texts = ",".join(content)
        return texts

    def _parse_slide(self, slide):
        """
        解析单个幻灯片的文本和表格。
        :param slide: PPTX 幻灯片对象
        :return: 包含文本和表格数据的元组
        """
        # 提取文本
        text = ""
        for shape in slide.shapes:
            if not shape.has_text_frame:
                continue
            for paragraph in shape.text_frame.paragraphs:
                for run in paragraph.runs:
                    text += run.text + " "

        # 提取表格
        tables = []
        for shape in slide.shapes:
            if not shape.has_table:
                continue
            for row in shape.table.rows:
                row_data = [cell.text for cell in row.cells]
                tables.append(", ".join(row_data))

        result = "".join([text.strip()] + tables)
        return [result]


if __name__ == '__main__':
    parser = PPTXParser()
    print(parser.parser('/home/wangyan/操作系统大会演示素材v0.2/团建策划资料/户外团建活动游戏(1).pptx'))



# /*
#  * Copyright 2024 KylinSoft Co., Ltd.
#  *
#  * This program is free software: you can redistribute it and/or modify it under
#  * the terms of the GNU General Public License as published by the Free Software
#  * Foundation, either version 3 of the License, or (at your option) any later
#  * version.
#  *
#  * This program is distributed in the hope that it will be useful, but WITHOUT
#  * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
#  * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
#  * details.
#  *
#  * You should have received a copy of the GNU General Public License along with
#  * this program. If not, see <https://www.gnu.org/licenses/>.
#  */

import subprocess
import os
from .base_parser import BaseParser
from .docx_parser import DOCXParser

"""
wps文件解析器,由于包的依赖问题，暂时不用
"""
class WPSParser(BaseParser):

    def convert_wps_to_docx(self, file_path):
        docx_file_path = file_path.replace(".wps", ".docx")
        subprocess.run(["unoconv", "-f", "docx", "--output", docx_file_path, file_path], check=True)
        print(f"文件已成功从 {file_path} 转换为 {docx_file_path}")
        return docx_file_path


    def parser(self, file_path) -> str:
        """
        解析wps文件
        """
        docx_file_path = self.convert_wps_to_docx(file_path)

        docx = DOCXParser()
        content_list = docx.parser_docx(docx_file_path)

        os.remove(docx_file_path)
        return content_list


if __name__ == "__main__":
    parser = WPSParser()
    print(parser.parser("/home/kylin/wangyan/qqq.wps"))



# /*
#  * Copyright 2024 KylinSoft Co., Ltd.
#  *
#  * This program is free software: you can redistribute it and/or modify it under
#  * the terms of the GNU General Public License as published by the Free Software
#  * Foundation, either version 3 of the License, or (at your option) any later
#  * version.
#  *
#  * This program is distributed in the hope that it will be useful, but WITHOUT
#  * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
#  * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
#  * details.
#  *
#  * You should have received a copy of the GNU General Public License along with
#  * this program. If not, see <https://www.gnu.org/licenses/>.
#  */

from parser.base_parser import BaseParser
from parser.txt_parser import TXTParser
#from parser.image_parser import ImageParser
from parser.json_parser import JSONParser
from parser.html_parser import HTMLParser
from parser.xml_parser import XMLParser
from parser.md_parser import MDParser

class ParserFactory:
    def __init__(self) -> None:
        pass
    @staticmethod
    def create_parser(format_type: str) -> BaseParser:
        if format_type == "txt":
            return TXTParser()
        elif format_type == "pdf":
            from parser.pdf_parser import PDFParser
            return PDFParser()
        elif format_type == "docx":
            from parser.docx_parser import DOCXParser
            return DOCXParser()
        elif format_type == "pptx":
            from parser.pptx_parser import PPTXParser
            return PPTXParser()
        elif format_type == "doc" or format_type == "wps" or format_type == "ppt" or \
            format_type == "uof" or \
            format_type == "uop" or \
            format_type == "uot" or \
            format_type == "uos" or \
            format_type == "dot" or \
            format_type == "pps" or \
            format_type == "dps" or \
            format_type == "et" or \
            format_type == "xls" :
            from parser.office_parser import OfficeParser
            return OfficeParser()
        # elif format_type == "jpg" or format_type == "png" or format_type == "jpeg":
        #     return ImageParser()
        elif format_type == "json":
            return JSONParser()
        elif format_type == "html":
            return HTMLParser()
        elif format_type == "xml":
            return XMLParser()
        elif format_type == "md":
            return MDParser()
        elif format_type == "xlsx":
            from parser.excel_parser import EXCELParser
            return EXCELParser()
        else:
            raise ValueError(f"Unsupported format: {format_type}")

if __name__ == "__main__":
    import os
    filepath = "/usr/share/kylin-user-guide/data/guide/accessories-text-editor/zh_CN/index.md"
    file_extension = os.path.splitext(filepath)[1]
    file_format = file_extension[1:]
    parser = ParserFactory.create_parser(file_format)
    print(parser.parser(filepath))


# Generated by the gRPC Python protocol compiler plugin. DO NOT EDIT!
"""Client and server classes corresponding to protobuf-defined services."""
import grpc

import document_pb2 as document__pb2


class DocumentServiceStub(object):
    """定义服务
    """

    def __init__(self, channel):
        """Constructor.

        Args:
            channel: A grpc.Channel.
        """
        self.SyncChunkDocument = channel.unary_unary(
                '/document.DocumentService/SyncChunkDocument',
                request_serializer=document__pb2.ChunkRequest.SerializeToString,
                response_deserializer=document__pb2.ChunkedResponse.FromString,
                )
        self.AsyncChunkDocument = channel.unary_unary(
                '/document.DocumentService/AsyncChunkDocument',
                request_serializer=document__pb2.ChunkRequest.SerializeToString,
                response_deserializer=document__pb2.ChunkedResponse.FromString,
                )
        self.StreamChunkDocument = channel.unary_stream(
                '/document.DocumentService/StreamChunkDocument',
                request_serializer=document__pb2.ChunkRequest.SerializeToString,
                response_deserializer=document__pb2.ChunkedResponse.FromString,
                )


class DocumentServiceServicer(object):
    """定义服务
    """

    def SyncChunkDocument(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def AsyncChunkDocument(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def StreamChunkDocument(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')


def add_DocumentServiceServicer_to_server(servicer, server):
    rpc_method_handlers = {
            'SyncChunkDocument': grpc.unary_unary_rpc_method_handler(
                    servicer.SyncChunkDocument,
                    request_deserializer=document__pb2.ChunkRequest.FromString,
                    response_serializer=document__pb2.ChunkedResponse.SerializeToString,
            ),
            'AsyncChunkDocument': grpc.unary_unary_rpc_method_handler(
                    servicer.AsyncChunkDocument,
                    request_deserializer=document__pb2.ChunkRequest.FromString,
                    response_serializer=document__pb2.ChunkedResponse.SerializeToString,
            ),
            'StreamChunkDocument': grpc.unary_stream_rpc_method_handler(
                    servicer.StreamChunkDocument,
                    request_deserializer=document__pb2.ChunkRequest.FromString,
                    response_serializer=document__pb2.ChunkedResponse.SerializeToString,
            ),
    }
    generic_handler = grpc.method_handlers_generic_handler(
            'document.DocumentService', rpc_method_handlers)
    server.add_generic_rpc_handlers((generic_handler,))


 # This class is part of an EXPERIMENTAL API.
class DocumentService(object):
    """定义服务
    """

    @staticmethod
    def SyncChunkDocument(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/document.DocumentService/SyncChunkDocument',
            document__pb2.ChunkRequest.SerializeToString,
            document__pb2.ChunkedResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def AsyncChunkDocument(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/document.DocumentService/AsyncChunkDocument',
            document__pb2.ChunkRequest.SerializeToString,
            document__pb2.ChunkedResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def StreamChunkDocument(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_stream(request, target, '/document.DocumentService/StreamChunkDocument',
            document__pb2.ChunkRequest.SerializeToString,
            document__pb2.ChunkedResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)



# /*
#  * Copyright 2024 KylinSoft Co., Ltd.
#  *
#  * This program is free software: you can redistribute it and/or modify it under
#  * the terms of the GNU General Public License as published by the Free Software
#  * Foundation, either version 3 of the License, or (at your option) any later
#  * version.
#  *
#  * This program is distributed in the hope that it will be useful, but WITHOUT
#  * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
#  * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
#  * details.
#  *
#  * You should have received a copy of the GNU General Public License along with
#  * this program. If not, see <https://www.gnu.org/licenses/>.
#  */

import fitz  # PyMuPDF
from .base_parser import BaseParser

class PDFParser(BaseParser):
    def parser(self, file_path) -> list:
        """
        同时解析PDF文件中的文本和表格，并将它们合并到一个列表中。
        :return: 包含每页内容（文本和表格）的列表
        """
        pdf = fitz.open(file_path)
        content = []

        for page_num in range(pdf.page_count):
            page = pdf.load_page(page_num)
            results = self._parse_page(page)
            if results == ['']:
                continue
            else:
                content.extend(results)
        texts = ",".join(content)
        return texts

    def _parse_page(self, page) -> list:
        """
        解析单个页面的文本和表格。
        :param page: PyMuPDF 页面对象
        :return: 包含文本和表格数据的列表
        """
        # 提取文本
        text = page.get_text("text").strip()
        text = ''.join(text)

        # 提取表格
        tables = []
        # blocks = page.get_text("dict")["blocks"]
        # for block in blocks:
        #     if block["type"] == 0:
        #         table_data = []
        #         for line in block["lines"]:
        #             row_data = [span["text"] for span in line["spans"]]
        #             table_data.append(", ".join(row_data))
        #         tables.extend(table_data)
        # 将文本和表格合并到一个列表中
        combined_data = [text] + tables
        result = "".join(combined_data)
        return [result]

if __name__ == '__main__':
    DocxParser = PDFParser()
    paras = DocxParser.parser('/home/wangyan/操作系统大会演示素材v0.2/团建策划资料/怀柔1日-水上闯关团建方案-pptx(2).pdf')
    print(paras)



# /*
#  * Copyright 2024 KylinSoft Co., Ltd.
#  *
#  * This program is free software: you can redistribute it and/or modify it under
#  * the terms of the GNU General Public License as published by the Free Software
#  * Foundation, either version 3 of the License, or (at your option) any later
#  * version.
#  *
#  * This program is distributed in the hope that it will be useful, but WITHOUT
#  * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
#  * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
#  * details.
#  *
#  * You should have received a copy of the GNU General Public License along with
#  * this program. If not, see <https://www.gnu.org/licenses/>.
#  */

from bs4 import BeautifulSoup
from .base_parser import BaseParser

class HTMLParser(BaseParser):

    def parser(self, file_path) -> list:
        """
        :param file_path: 文件路径
        :return: 按行保存的文本内容列表
        """
        with open(file_path, 'r', encoding='utf-8') as file:
            html_content = file.read()

        soup = BeautifulSoup(html_content, 'html.parser')
        text = soup.get_text(separator=' ')
        text = ''.join(text.split())

        return [text]
    def parser_tag(self, file_path, tag_name, attrs=None):
        with open(file_path, 'r', encoding='utf-8') as file:
            html_content = file.read()
        soup = BeautifulSoup(html_content, 'html.parser')
        tags = soup.find_all(tag_name, attrs=attrs)
        return [tag.get_text(strip=True) for tag in tags]

if __name__ == '__main__':
    file_path = '/home/kylin/wangyan/test-search/touchgesture.xml'
    html_parser = HTMLParser()
    html_content = html_parser.parser(file_path)
    # print(html_parser.parser_tag(file_path, 'keys'))
    print(html_content)


# /*
#  * Copyright 2024 KylinSoft Co., Ltd.
#  *
#  * This program is free software: you can redistribute it and/or modify it under
#  * the terms of the GNU General Public License as published by the Free Software
#  * Foundation, either version 3 of the License, or (at your option) any later
#  * version.
#  *
#  * This program is distributed in the hope that it will be useful, but WITHOUT
#  * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
#  * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
#  * details.
#  *
#  * You should have received a copy of the GNU General Public License along with
#  * this program. If not, see <https://www.gnu.org/licenses/>.
#  */

from openpyxl import load_workbook
from .base_parser import BaseParser

class EXCELParser(BaseParser):

    def parser(self, file_path) -> list:
        workbook = load_workbook(filename=file_path)
        text = []
        for sheet_name in workbook.sheetnames:
            sheet = workbook[sheet_name]
            for row in sheet.iter_rows(values_only=True):
                for cell_value in row:
                    if cell_value is not None:
                        text.append(str(cell_value))
        result = "".join(text)
        return [result]

if __name__ == '__main__':
    file_path = '/home/kylin/wangyan/test-search/向量数据库.xlsx'
    excel_parser = EXCELParser()
    text = excel_parser.parser(file_path)
    print(text)



# /*
#  * Copyright 2024 KylinSoft Co., Ltd.
#  *
#  * This program is free software: you can redistribute it and/or modify it under
#  * the terms of the GNU General Public License as published by the Free Software
#  * Foundation, either version 3 of the License, or (at your option) any later
#  * version.
#  *
#  * This program is distributed in the hope that it will be useful, but WITHOUT
#  * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
#  * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
#  * details.
#  *
#  * You should have received a copy of the GNU General Public License along with
#  * this program. If not, see <https://www.gnu.org/licenses/>.
#  */

import grpc
import json
import document_pb2_grpc
import document_pb2
import time
import os
import logging
import asyncio

# 配置日志文件的位置
log_file_path = "logs/parse_client.log"
# 创建日志目录（如果不存在）
os.makedirs(os.path.dirname(log_file_path), exist_ok=True)
# 配置日志记录器
logging.basicConfig(
    level=logging.INFO,  # 日志级别
    format="%(asctime)s - %(levelname)s - %(message)s",  # 日志格式
    handlers=[
        logging.FileHandler(log_file_path),  # 输出到文件
        logging.StreamHandler()  # 同时输出到控制台
    ]
)
logging.info("Logging is set up. Starting the client...")

def run_sync(client, files):
    # 构造包含多个文件及其解析类型的请求
    request = document_pb2.ParseRequest(files=files)
    # 调用同步解析接口
    response = client.SyncParseDocument(request)
    print(response.results)

async def run_async(client, files):
    request = document_pb2.ParseRequest(files=files)
    logging.info(f"Sending async request for files: {files}")
    # responses = client.AsyncParseDocument(request)
    async for response in client.AsyncParseDocument(request):
        print(f"Async response: {response}")
        logging.info(f"Received async response: {response}")
        for field in response.DESCRIPTOR.fields:
                value = getattr(response, field.name)
                logging.info(f"  {field.name}: {value}")


def main():
    with grpc.insecure_channel('localhost:50051') as channel_sync, grpc.insecure_channel('localhost:50052') as channel_async:
        sync_client = document_pb2_grpc.DocumentServiceStub(channel_sync)
        async_client = document_pb2_grpc.DocumentServiceStub(channel_async)

        # 示例：文件路径与解析类型的映射
        files = {
            '/home/mayuan/codeyard/DocumentService/pdftest.pdf': 'TextChunker',
            '/home/mayuan/codeyard/DocumentService/embedding.pdf': 'TextChunker',
            '/home/mayuan/codeyard/DocumentService/data/金融知识库.pdf': 'TextChunker',
            '/home/mayuan/codeyard/DocumentService/data/麒麟软件工会经费报销单.docx': 'TextChunker'
        }

        # 运行同步请求
        run_sync(async_client, files)
async def main():
    # 初始化 gRPC 异步通道
    channel = grpc.aio.insecure_channel('localhost:50052')
    # 检查通道是否准备好
    try:
        await channel.channel_ready()
        logging.info("Channel is ready")
    except grpc.FutureTimeoutError:
        logging.error("Channel is not ready. Check if the server is running and accessible.")
        return
    stub = document_pb2_grpc.DocumentServiceStub(channel)
    # 文件列表
    files = {
            '/home/mayuan/codeyard/DocumentService/pdftest.pdf': 'TextChunker',
            '/home/mayuan/codeyard/DocumentService/embedding.pdf': 'TextChunker',
            '/home/mayuan/codeyard/DocumentService/data/金融知识库.pdf': 'TextChunker',
            '/home/mayuan/codeyard/DocumentService/data/麒麟软件工会经费报销单.docx': 'TextChunker'
        }

    # 调用异步函数
    await run_async(stub, files)

if __name__ == '__main__':
    time_start = time.time()
    # main()
    asyncio.run(main())
    time_end = time.time()
    print(time_end - time_start)



# /*
#  * Copyright 2024 KylinSoft Co., Ltd.
#  *
#  * This program is free software: you can redistribute it and/or modify it under
#  * the terms of the GNU General Public License as published by the Free Software
#  * Foundation, either version 3 of the License, or (at your option) any later
#  * version.
#  *
#  * This program is distributed in the hope that it will be useful, but WITHOUT
#  * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
#  * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
#  * details.
#  *
#  * You should have received a copy of the GNU General Public License along with
#  * this program. If not, see <https://www.gnu.org/licenses/>.
#  */

import re
from base_chunker import BaseChunker

class TextChunker(BaseChunker):
    def is_english(self,text):
        return all(ord(c) < 128 for c in text)
    def apply_regex(self, text) -> str:
        newline_regex = r"\n{3,}"
        space_regex = r"\s+"
        double_newline_regex = r"\n\n"
        single_punctuation_regex = r"([;；.!?。！？\?])([^”’'])"
        ellipsis_regex = r"(\\.\\.\\.)([^\"’”」』])"
        chinese_ellipsis_regex = r"(…{2})([^\"’”」』])"
        end_quote_punctuation_regex = r"([;；!?。！？\?][\"’”」』]{0,2})([^;；!?，。！？\?])"

        # 处理多余的换行符
        text = re.sub(newline_regex, "\n", text)

        # 标准化空格
        text = re.sub(space_regex, " ", text)

        # 移除双换行
        text = re.sub(double_newline_regex, "", text)

        # 处理标点符号后的换行
        text = re.sub(single_punctuation_regex, r"\1\n\2", text)

        # 处理省略号后的换行
        text = re.sub(ellipsis_regex, r"\1\n\2", text)

        # 处理中文省略号后的换行
        text = re.sub(chinese_ellipsis_regex, r"\1\n\2", text)

        # 处理结尾引号后的标点
        text = re.sub(end_quote_punctuation_regex, r"\1\n\2", text)

        # 删除所有空白字符
        if self.is_english(text):
            text = re.sub(r"\s+", " ", text)
        else:
            text = re.sub(r"\s+", "", text)

        return text

    def split_sentence_into_chunks(self,sentence, max_chunk_size=100):
        punctuation = r'.+?[.?!,;，。！？：]+'

        chunks = []
        current_chunk = ""
        for part in re.split(f"({punctuation})", sentence):
            part = part.strip()
            if not part:
                continue
            if len(current_chunk) + len(part) <= max_chunk_size:
                current_chunk += part
            else:
                if current_chunk:
                    chunks.append(current_chunk.strip())
                current_chunk = part
        if current_chunk:
            chunks.append(current_chunk.strip())

        return chunks
    
    def split_sentence_length(self, sentence, max_chunk_size=100):
        sentence_list = []
        for sen in sentence:
            sentence_list.extend([sen[i:i+max_chunk_size] for i in range(0, len(sen), max_chunk_size)])
        return sentence_list
    
    def segment_tokenize(self, texts, max_length) -> list:
        if isinstance(texts, str):
            texts = [texts]  # 将字符串转换为列表
        result_list = []
        for text in texts:
            sentence = self.split_sentence_into_chunks(sentence=text,max_chunk_size=max_length)
            sentence_list = self.split_sentence_length(sentence=sentence,max_chunk_size=max_length)
            result_list.extend(sentence_list)
        return result_list

# if __name__ == "__main__":
#     text = ""
#     textchunker = TextChunker()
#     textchunker.segment_tokenize(text, 500)